@@ -29,17 +29,11 @@
 #include "knotification.h"
 #include "knotificationmanager_p.h"
 
-#include <kwindowsystem.h>
-
 #include <QCoreApplication>
 
-#include <QMap>
 #include <QPixmap>
-#include <QPointer>
-#include <QLabel>
 #include <QTimer>
 #include <QTabWidget>
-#include <QFile>
 #include <QStringList>
 #include <QUrl>
 
@@ -59,18 +53,21 @@ struct Q_DECL_HIDDEN KNotification::Private {
     NotificationFlags flags;
     QString componentName;
     QList<QUrl> urls;
+    KNotification::Urgency urgency;
     QVariantMap hints;
 
     QTimer updateTimer;
     bool needUpdate;
 
-    Private() : id(-1), ref(0), widget(nullptr), needUpdate(false) {}
+    Private() : id(-1), ref(0), widget(nullptr), urgency(KNotification::DefaultUrgency), needUpdate(false) {}
+#if KNOTIFICATIONS_BUILD_DEPRECATED_SINCE(5, 67)
     /**
      * recursive function that raise the widget. @p w
      *
      * @see raiseWidget()
      */
     static void raiseWidget(QWidget *w);
+#endif
 };
 
 KNotification::KNotification(const QString &eventId, QWidget *parent, const NotificationFlags &flags) :
@@ -257,7 +254,15 @@ KNotification::NotificationFlags KNotification::flags() const
 
 void KNotification::setFlags(const NotificationFlags &flags)
 {
+    if (d->flags == flags) {
+        return;
+    }
+
+    d->needUpdate = true;
     d->flags = flags;
+    if (d->id >= 0) {
+        d->updateTimer.start();
+    }
 }
 
 void KNotification::setComponentName(const QString &c)
@@ -272,7 +277,33 @@ QList<QUrl> KNotification::urls() const
 
 void KNotification::setUrls(const QList<QUrl> &urls)
 {
+    if (d->urls == urls) {
+        return;
+    }
+
+    d->needUpdate = true;
     d->urls = urls;
+    if (d->id >= 0) {
+        d->updateTimer.start();
+    }
+}
+
+KNotification::Urgency KNotification::urgency() const
+{
+    return d->urgency;
+}
+
+void KNotification::setUrgency(Urgency urgency)
+{
+    if (d->urgency == urgency) {
+        return;
+    }
+
+    d->needUpdate = true;
+    d->urgency = urgency;
+    if (d->id >= 0) {
+        d->updateTimer.start();
+    }
 }
 
 void KNotification::activate(unsigned int action)
@@ -280,6 +311,7 @@ void KNotification::activate(unsigned int action)
     switch (action) {
     case 0:
         emit activated();
+        emit defaultActivated();
         break;
     case 1:
         emit action1Activated();
@@ -311,6 +343,7 @@ void KNotification::close()
     }
 }
 
+#if KNOTIFICATIONS_BUILD_DEPRECATED_SINCE(5, 67)
 void KNotification::raiseWidget()
 {
     if (!d->widget) {
@@ -319,16 +352,15 @@ void KNotification::raiseWidget()
 
     Private::raiseWidget(d->widget);
 }
+#endif
 
+#if KNOTIFICATIONS_BUILD_DEPRECATED_SINCE(5, 67)
 void KNotification::Private::raiseWidget(QWidget *w)
 {
     //TODO  this function is far from finished.
     if (w->isTopLevel()) {
         w->raise();
-        KWindowSystem::activateWindow(w->winId());
-#if defined(Q_OS_MACOS)
         w->activateWindow();
-#endif
     } else {
         QWidget *pw = w->parentWidget();
         raiseWidget(pw);
@@ -338,13 +370,14 @@ void KNotification::Private::raiseWidget(QWidget *w)
         }
     }
 }
+#endif
 
 static QString defaultComponentName()
 {
-#ifndef Q_OS_ANDROID
-    return QStringLiteral("plasma_workspace");
-#else
+#if defined(Q_OS_ANDROID)
     return QStringLiteral("android_defaults");
+#else
+    return QStringLiteral("plasma_workspace");
 #endif
 }
 
