@@ -27,6 +27,7 @@
 #include <QList>
 #include <QPair>
 #include <QUrl>
+#include <QVariant>
 
 class QWidget;
 
@@ -70,6 +71,7 @@ class QWidget;
  * \section file The global config file
  * Your application should install a file called <em>knotifications5/appname.notifyrc</em>
  * in a QStandardPaths::GenericDataLocation directory.
+ * On Android, this path is <em>qrc:/knotifications5/</em>.
  *
  * The filename must either match QCoreApplication::applicationName or be specified as the
  * component name to the KNotification object.
@@ -254,7 +256,7 @@ public:
         /**
          * Sends a hint to Plasma to skip grouping for this notification
          *
-         * @since: 5.18
+         * @since 5.18
          */
         SkipGrouping = 0x10,
 
@@ -411,7 +413,7 @@ public:
      */
     QPixmap pixmap() const;
     /**
-     * Set the pixmap that will be shown in the popup.
+     * Set the pixmap that will be shown in the popup. If you want to use an icon from the icon theme use setIconName instead.
      *
      * @param pix the pixmap
      */
@@ -565,6 +567,13 @@ Q_SIGNALS:
      * Emitted only when the default activation has occurred
      */
     void activated();
+
+    /**
+     * Emitted when the default action has been activated.
+     * @since 5.67
+     */
+    void defaultActivated();
+
     /**
      * Emitted when an action has been activated.
      *
@@ -617,11 +626,15 @@ public Q_SLOTS:
      */
     void close();
 
+#if KNOTIFICATIONS_ENABLE_DEPRECATED_SINCE(5, 67)
     /**
      * @brief Raise the widget.
      * This will change the desktop, activate the window, and the tab if needed.
+     * @deprecated since 5.67, use QWindow raise + requestActivate instead.
      */
+    KNOTIFICATIONS_DEPRECATED_VERSION(5, 67, "Use QWindow raise + requestActivate instead")
     void raiseWidget();
+#endif
 
     /**
      * The notification will automatically be closed if all presentations are finished.
