@@ -22,9 +22,6 @@
 #include "knotification.h"
 
 #include <QHash>
-#include <QWidget>
-#include <QPointer>
-#include <QBuffer>
 #include <QFileInfo>
 #include <KPluginLoader>
 #include <KPluginMetaData>
@@ -40,11 +37,16 @@
 #include "notifybylogfile.h"
 #include "notifybytaskbar.h"
 #include "notifybyexecute.h"
-#ifndef Q_OS_ANDROID
+
+#if defined(Q_OS_ANDROID)
+#include "notifybyandroid.h"
+#elif defined(Q_OS_MACOS)
+#include "notifybymacosnotificationcenter.h"
+#elif defined(Q_OS_WIN)
+#include "notifybysnore.h"
+#else
 #include "notifybypopup.h"
 #include "notifybyportal.h"
-#else
-#include "notifybyandroid.h"
 #endif
 #include "debug_p.h"
 
@@ -91,10 +93,10 @@ KNotificationManager::KNotificationManager()
     d->notifyPlugins.clear();
 
 #ifdef QT_DBUS_LIB
-    const bool inSandbox = QFileInfo::exists(QLatin1String("/.flatpak-info")) || qEnvironmentVariableIsSet("SNAP");
+     const bool inSandbox = QFileInfo::exists(QLatin1String("/.flatpak-info")) || qEnvironmentVariableIsSet("SNAP");
 
-    if (inSandbox) {
-        QDBusConnectionInterface *interface = QDBusConnection::sessionBus().interface();
+     if (inSandbox) {
+         QDBusConnectionInterface *interface = QDBusConnection::sessionBus().interface();
         d->portalDBusServiceExists = interface->isServiceRegistered(QStringLiteral("org.freedesktop.portal.Desktop"));
     }
 
@@ -133,24 +135,27 @@ KNotificationPlugin *KNotificationManager::pluginForAction(const QString &action
     // We have a series of built-ins up first, and fall back to trying
     // to instantiate an externally supplied plugin.
     if (action == QLatin1String("Popup")) {
-#ifndef Q_OS_ANDROID
+#if defined(Q_OS_ANDROID)
+            plugin = new NotifyByAndroid(this);
+#elif defined(Q_OS_WIN)
+            plugin = new NotifyBySnore(this);
+#elif defined(Q_OS_MACOS)
+            plugin = new NotifyByMacOSNotificationCenter(this);
+#else
             if (d->portalDBusServiceExists) {
                 plugin = new NotifyByPortal(this);
             } else {
                 plugin = new NotifyByPopup(this);
-            }
-#else
-        plugin = new NotifyByAndroid(this);
+        }
 #endif
-
         addPlugin(plugin);
     } else if (action == QLatin1String("Taskbar")) {
         plugin = new NotifyByTaskbar(this);
         addPlugin(plugin);
     } else if (action == QLatin1String("Sound")) {
 #if defined(HAVE_PHONON4QT5) || defined(HAVE_CANBERRA)
-        plugin = new NotifyByAudio(this);
-        addPlugin(plugin);
+            plugin = new NotifyByAudio(this);
+            addPlugin(plugin);
 #endif
     } else if (action == QLatin1String("Execute")) {
         plugin = new NotifyByExecute(this);
@@ -187,7 +192,7 @@ KNotificationPlugin *KNotificationManager::pluginForAction(const QString &action
                 // instantiate the plugin, unless we already got a match.
                 // TODO KF6: Require X-KDE-KNotification-OptionName be set and
                 // reject plugins without it.
-                if (rawData.contains(QStringLiteral("X-KDE-KNotification-OptionName"))) {
+                if (rawData.contains(QLatin1String("X-KDE-KNotification-OptionName"))) {
                     if (rawData.value(QStringLiteral("X-KDE-KNotification-OptionName")) == action) {
                         pluginFound = true;
                     } else {
