@@ -27,12 +27,10 @@
 #include <QBitmap>
 #include <QScreen>
 #include <QLabel>
-#include <QLayout>
 #include <QBoxLayout>
 #include <QMouseEvent>
 #include <QPainter>
 #include <QPainterPath>
-#include <QPolygonF>
 #include <QStyle>
 #include <QTimer>
 #include <QToolTip>
@@ -43,7 +41,9 @@
 #include <netwm.h>
 #endif
 
+#if HAVE_KWINDOWSYSTEM
 #include <kwindowinfo.h>
+#endif
 
 static const int DEFAULT_POPUP_TYPE = KPassivePopup::Boxed;
 static const int DEFAULT_POPUP_TIME = 6 * 1000;
@@ -235,8 +235,8 @@ public:
 
     QRect desktopRectForPoint(const QPoint &point)
     {
-        QList<QScreen*> screens = QGuiApplication::screens();
-        Q_FOREACH(const QScreen *screen, screens) {
+        const QList<QScreen*> screens = QGuiApplication::screens();
+        for(const QScreen *screen : screens) {
             if (screen->geometry().contains(point)) {
                 return screen->geometry();
             }
@@ -287,8 +287,8 @@ void KPassivePopup::setView(QWidget *child)
     delete d->topLayout;
     d->topLayout = new QVBoxLayout(this);
     if (d->popupStyle == Balloon) {
-        const int marginHint = style()->pixelMetric(QStyle::PM_DefaultChildMargin);
-        d->topLayout->setMargin(2 * marginHint);
+        const int marginHint = 2 * style()->pixelMetric(QStyle::PM_DefaultChildMargin);
+        d->topLayout->setContentsMargins(marginHint, marginHint, marginHint, marginHint);
     }
     d->topLayout->addWidget(d->msgView);
     d->topLayout->activate();
@@ -308,13 +308,13 @@ QWidget *KPassivePopup::standardView(const QString &caption,
 {
     QWidget *top = new QWidget(parent ? parent : this);
     QVBoxLayout *vb = new QVBoxLayout(top);
-    vb->setMargin(0);
+    vb->setContentsMargins(0, 0, 0, 0);
     top->setLayout(vb);
 
     QHBoxLayout *hb = nullptr;
     if (!icon.isNull()) {
         hb = new QHBoxLayout;
-        hb->setMargin(0);
+        hb->setContentsMargins(0, 0, 0, 0);
         vb->addLayout(hb);
         d->ttlIcon = new QLabel(top);
         d->ttlIcon->setPixmap(icon);
@@ -473,12 +473,14 @@ void KPassivePopup::positionSelf()
                 target = widget->geometry();
             }
         }
+#if HAVE_KWINDOWSYSTEM
         if (target.isNull()) {
             KWindowInfo info(d->window, NET::WMGeometry);
             if (info.valid()) {
                 target = info.geometry();
             }
         }
+#endif
     }
     if (target.isNull()) {
         target = QRect(defaultLocation(), QSize(0, 0));
