@@ -20,11 +20,10 @@
 
 #include "kstatusnotifieritem.h"
 #include "kstatusnotifieritemprivate_p.h"
-#include "kstatusnotifieritemdbus_p.h"
 #include "debug_p.h"
+#include "config-knotifications.h"
+
 
-#include <QDBusConnection>
-#include <QtEndian>
 #include <QMessageBox>
 #include <QPixmap>
 #include <QImage>
@@ -38,20 +37,25 @@
 #include <QtMac>
 #endif
 
+#ifdef QT_DBUS_LIB
+#include "kstatusnotifieritemdbus_p.h"
+
+#include <QDBusConnection>
+
+#if HAVE_DBUSMENUQT
+#include <dbusmenuexporter.h>
+#endif //HAVE_DBUSMENUQT
+#endif
+
+#include <QTimer>
 #include <kwindowinfo.h>
 #include <kwindowsystem.h>
 
 #include <cstdlib>
 
-#include <config-knotifications.h>
-
 static const char s_statusNotifierWatcherServiceName[] = "org.kde.StatusNotifierWatcher";
 static const int s_legacyTrayIconSize = 24;
 
-#if HAVE_DBUSMENUQT
-#include <dbusmenuexporter.h>
-#endif //HAVE_DBUSMENUQT
-
 KStatusNotifierItem::KStatusNotifierItem(QObject *parent)
     : QObject(parent),
       d(new KStatusNotifierItemPrivate(this))
@@ -68,8 +72,10 @@ KStatusNotifierItem::KStatusNotifierItem(const QString &id, QObject *parent)
 
 KStatusNotifierItem::~KStatusNotifierItem()
 {
+#ifdef QT_DBUS_LIB
     delete d->statusNotifierWatcher;
     delete d->notificationsClient;
+#endif
     delete d->systemTrayIcon;
     if (!qApp->closingDown()) {
         delete d->menu;
@@ -113,8 +119,10 @@ void KStatusNotifierItem::setStatus(const ItemStatus status)
     }
 
     d->status = status;
-    emit d->statusNotifierItemDBus->NewStatus(QString::fromLatin1(metaObject()->enumerator(metaObject()->indexOfEnumerator("ItemStatus")).valueToKey(d->status)));
 
+#ifdef QT_DBUS_LIB
+    emit d->statusNotifierItemDBus->NewStatus(QString::fromLatin1(metaObject()->enumerator(metaObject()->indexOfEnumerator("ItemStatus")).valueToKey(d->status)));
+#endif
     if (d->systemTrayIcon) {
         d->syncLegacySystemTrayIcon();
     }
@@ -128,9 +136,14 @@ void KStatusNotifierItem::setIconByName(const QString &name)
         return;
     }
 
-    d->serializedIcon = KDbusImageVector();
+
     d->iconName = name;
+
+#ifdef QT_DBUS_LIB
+    d->serializedIcon = KDbusImageVector();
     emit d->statusNotifierItemDBus->NewIcon();
+#endif
+
     if (d->systemTrayIcon) {
         d->systemTrayIcon->setIcon(QIcon::fromTheme(name));
     }
@@ -148,8 +161,11 @@ void KStatusNotifierItem::setIconByPixmap(const QIcon &icon)
     }
 
     d->iconName.clear();
+
+#ifdef QT_DBUS_LIB
     d->serializedIcon = d->iconToVector(icon);
     emit d->statusNotifierItemDBus->NewIcon();
+#endif
 
     d->icon = icon;
     if (d->systemTrayIcon) {
@@ -169,7 +185,9 @@ void KStatusNotifierItem::setOverlayIconByName(const QString &name)
     }
 
     d->overlayIconName = name;
+#ifdef QT_DBUS_LIB
     emit d->statusNotifierItemDBus->NewOverlayIcon();
+#endif
     if (d->systemTrayIcon) {
         QPixmap iconPixmap = QIcon::fromTheme(d->iconName).pixmap(s_legacyTrayIconSize, s_legacyTrayIconSize);
         if (!name.isEmpty()) {
@@ -194,8 +212,11 @@ void KStatusNotifierItem::setOverlayIconByPixmap(const QIcon &icon)
     }
 
     d->overlayIconName.clear();
+
+#ifdef QT_DBUS_LIB
     d->serializedOverlayIcon = d->iconToVector(icon);
     emit d->statusNotifierItemDBus->NewOverlayIcon();
+#endif
 
     d->overlayIcon = icon;
     if (d->systemTrayIcon) {
@@ -222,9 +243,12 @@ void KStatusNotifierItem::setAttentionIconByName(const QString &name)
         return;
     }
 
-    d->serializedAttentionIcon = KDbusImageVector();
     d->attentionIconName = name;
+
+#ifdef QT_DBUS_LIB
+    d->serializedAttentionIcon = KDbusImageVector();
     emit d->statusNotifierItemDBus->NewAttentionIcon();
+#endif
 }
 
 QString KStatusNotifierItem::attentionIconName() const
@@ -239,9 +263,12 @@ void KStatusNotifierItem::setAttentionIconByPixmap(const QIcon &icon)
     }
 
     d->attentionIconName.clear();
-    d->serializedAttentionIcon = d->iconToVector(icon);
     d->attentionIcon = icon;
+
+#ifdef QT_DBUS_LIB
+    d->serializedAttentionIcon = d->iconToVector(icon);
     emit d->statusNotifierItemDBus->NewAttentionIcon();
+#endif
 }
 
 QIcon KStatusNotifierItem::attentionIconPixmap() const
@@ -260,7 +287,9 @@ void KStatusNotifierItem::setAttentionMovieByName(const QString &name)
     delete d->movie;
     d->movie = nullptr;
 
+#ifdef QT_DBUS_LIB
     emit d->statusNotifierItemDBus->NewAttentionIcon();
+#endif
 
     if (d->systemTrayIcon) {
         d->movie = new QMovie(d->movieName);
@@ -313,7 +342,6 @@ void KStatusNotifierItem::setToolTip(const QString &iconName, const QString &tit
         return;
     }
 
-    d->serializedToolTipIcon = KDbusImageVector();
     d->toolTipIconName = iconName;
 
     d->toolTipTitle = title;
@@ -322,7 +350,11 @@ void KStatusNotifierItem::setToolTip(const QString &iconName, const QString &tit
 //     }
     setTrayToolTip(d->systemTrayIcon, title, subTitle);
     d->toolTipSubTitle = subTitle;
+
+#ifdef QT_DBUS_LIB
+    d->serializedToolTipIcon = KDbusImageVector();
     emit d->statusNotifierItemDBus->NewToolTip();
+#endif
 }
 
 void KStatusNotifierItem::setToolTip(const QIcon &icon, const QString &title, const QString &subTitle)
@@ -334,7 +366,6 @@ void KStatusNotifierItem::setToolTip(const QIcon &icon, const QString &title, co
     }
 
     d->toolTipIconName.clear();
-    d->serializedToolTipIcon = d->iconToVector(icon);
     d->toolTipIcon = icon;
 
     d->toolTipTitle = title;
@@ -344,7 +375,10 @@ void KStatusNotifierItem::setToolTip(const QIcon &icon, const QString &title, co
     setTrayToolTip(d->systemTrayIcon, title, subTitle);
 
     d->toolTipSubTitle = subTitle;
+#ifdef QT_DBUS_LIB
+    d->serializedToolTipIcon = d->iconToVector(icon);
     emit d->statusNotifierItemDBus->NewToolTip();
+#endif
 }
 
 void KStatusNotifierItem::setToolTipIconByName(const QString &name)
@@ -353,9 +387,11 @@ void KStatusNotifierItem::setToolTipIconByName(const QString &name)
         return;
     }
 
-    d->serializedToolTipIcon = KDbusImageVector();
     d->toolTipIconName = name;
+#ifdef QT_DBUS_LIB
+    d->serializedToolTipIcon = KDbusImageVector();
     emit d->statusNotifierItemDBus->NewToolTip();
+#endif
 }
 
 QString KStatusNotifierItem::toolTipIconName() const
@@ -370,9 +406,12 @@ void KStatusNotifierItem::setToolTipIconByPixmap(const QIcon &icon)
     }
 
     d->toolTipIconName.clear();
-    d->serializedToolTipIcon = d->iconToVector(icon);
     d->toolTipIcon = icon;
+
+#ifdef QT_DBUS_LIB
+    d->serializedToolTipIcon = d->iconToVector(icon);
     emit d->statusNotifierItemDBus->NewToolTip();
+#endif
 }
 
 QIcon KStatusNotifierItem::toolTipIconPixmap() const
@@ -387,7 +426,10 @@ void KStatusNotifierItem::setToolTipTitle(const QString &title)
     }
 
     d->toolTipTitle = title;
+
+#ifdef QT_DBUS_LIB
     emit d->statusNotifierItemDBus->NewToolTip();
+#endif
 //     if (d->systemTrayIcon) {
 //         d->systemTrayIcon->setToolTip(title);
 //     }
@@ -409,7 +451,9 @@ void KStatusNotifierItem::setToolTipSubTitle(const QString &subTitle)
 #ifdef Q_OS_MACOS
     setTrayToolTip(d->systemTrayIcon, d->toolTipTitle, subTitle);
 #endif
+#ifdef QT_DBUS_LIB
     emit d->statusNotifierItemDBus->NewToolTip();
+#endif
 }
 
 QString KStatusNotifierItem::toolTipSubTitle() const
@@ -450,7 +494,9 @@ void KStatusNotifierItem::setContextMenu(QMenu *menu)
     }
 
     d->menu = menu;
+    Qt::WindowFlags oldFlags = d->menu->windowFlags();
     d->menu->setParent(nullptr);
+    d->menu->setWindowFlags(oldFlags);
 }
 
 QMenu *KStatusNotifierItem::contextMenu() const
@@ -466,7 +512,7 @@ void KStatusNotifierItem::setAssociatedWidget(QWidget *associatedWidget)
 
         QObject::connect(KWindowSystem::self(), &KWindowSystem::windowAdded,
             d->associatedWidget, [this](WId id) {
-                if(d->associatedWidget->winId() == id) {
+                if(d->associatedWidget->winId() == id && d->associatedWidgetPos != QPoint(-1, -1)) {
                     d->associatedWidget->move(d->associatedWidgetPos);
                 }
             }
@@ -569,12 +615,6 @@ bool KStatusNotifierItem::standardActionsEnabled() const
 
 void KStatusNotifierItem::showMessage(const QString &title, const QString &message, const QString &icon, int timeout)
 {
-    if (!d->notificationsClient) {
-        d->notificationsClient = new org::freedesktop::Notifications(QStringLiteral("org.freedesktop.Notifications"), QStringLiteral("/org/freedesktop/Notifications"),
-                QDBusConnection::sessionBus());
-    }
-
-    uint id = 0;
 #ifdef Q_OS_MACOS
     if (d->systemTrayIcon) {
         // Growl is not needed anymore for QSystemTrayIcon::showMessage() since OS X 10.8
@@ -582,7 +622,27 @@ void KStatusNotifierItem::showMessage(const QString &title, const QString &messa
     } else
 #endif
     {
-        d->notificationsClient->Notify(d->title, id, icon, title, message, QStringList(), QVariantMap(), timeout);
+#ifdef QT_DBUS_LIB
+        if (!d->notificationsClient) {
+            d->notificationsClient = new org::freedesktop::Notifications(QStringLiteral("org.freedesktop.Notifications"), QStringLiteral("/org/freedesktop/Notifications"),
+                                                                         QDBusConnection::sessionBus());
+        }
+
+        uint id = 0;
+        QVariantMap hints;
+
+        QString desktopFileName = QGuiApplication::desktopFileName();
+        if (!desktopFileName.isEmpty()) {
+            // handle apps which set the desktopFileName property with filename suffix,
+            // due to unclear API dox (https://bugreports.qt.io/browse/QTBUG-75521)
+            if (desktopFileName.endsWith(QLatin1String(".desktop"))) {
+                desktopFileName.chop(8);
+            }
+            hints.insert(QStringLiteral("desktop-entry"), desktopFileName);
+        }
+
+        d->notificationsClient->Notify(d->title, id, icon, title, message, QStringList(), hints, timeout);
+#endif
     }
 }
 
@@ -600,15 +660,19 @@ void KStatusNotifierItem::activate(const QPoint &pos)
 #ifdef Q_OS_MACOS
         QtMac::setBadgeLabelText(QString());
 #endif
+#ifdef QT_DBUS_LIB
         emit d->statusNotifierItemDBus->NewStatus(QString::fromLatin1(metaObject()->enumerator(metaObject()->indexOfEnumerator("ItemStatus")).valueToKey(d->status)));
+#endif
     }
 
-    if (d->associatedWidget == d->menu) {
+#ifdef QT_DBUS_LIB
+    if (d->associatedWidget && d->associatedWidget == d->menu) {
         d->statusNotifierItemDBus->ContextMenu(pos.x(), pos.y());
         return;
     }
+#endif
 
-    if (d->menu->isVisible()) {
+    if (d->menu && d->menu->isVisible()) {
         d->menu->hide();
     }
 
@@ -637,9 +701,8 @@ bool KStatusNotifierItemPrivate::checkVisibility(QPoint pos, bool perform)
     }
 #endif
 #else
-    KWindowInfo info1(associatedWidget->winId(), NET::XAWMState | NET::WMState | NET::WMDesktop);
     // mapped = visible (but possibly obscured)
-    bool mapped = (info1.mappingState() == NET::Visible) && !info1.isMinimized();
+    const bool mapped = associatedWidget->isVisible() && !associatedWidget->isMinimized();
 
 //    - not mapped -> show, raise, focus
 //    - mapped
@@ -653,7 +716,8 @@ bool KStatusNotifierItemPrivate::checkVisibility(QPoint pos, bool perform)
         }
 
         return true;
-    } else {
+    } else if (QGuiApplication::platformName() == QLatin1String("xcb")) {
+        const KWindowInfo info1(associatedWidget->winId(), NET::XAWMState | NET::WMState | NET::WMDesktop);
         QListIterator< WId > it(KWindowSystem::stackingOrder());
         it.toBack();
         while (it.hasPrevious()) {
@@ -708,6 +772,12 @@ bool KStatusNotifierItemPrivate::checkVisibility(QPoint pos, bool perform)
             emit q->activateRequested(false, pos);
         }
 
+        return false;
+    } else {
+        if (perform) {
+            minimizeRestore(false); // hide
+            emit q->activateRequested(false, pos);
+        }
         return false;
     }
 #endif
@@ -722,7 +792,7 @@ bool KStatusNotifierItem::eventFilter(QObject *watched, QEvent *event)
         if (watched == d->menu &&
                 (event->type() == QEvent::WindowDeactivate || (event->type() == QEvent::MouseButtonRelease && static_cast<QMouseEvent *>(event)->button() == Qt::LeftButton))) {
             //put at the back of even queue to let the action activate anyways
-            QTimer::singleShot(0, this, SLOT(hideMenu()));
+            QTimer::singleShot(0, this, [this]() { d->hideMenu(); });
         }
     }
     return false;
@@ -740,8 +810,6 @@ KStatusNotifierItemPrivate::KStatusNotifierItemPrivate(KStatusNotifierItem *item
       menu(nullptr),
       associatedWidget(nullptr),
       titleAction(nullptr),
-      statusNotifierWatcher(nullptr),
-      notificationsClient(nullptr),
       systemTrayIcon(nullptr),
       hasQuit(false),
       onAllDesktops(false),
@@ -751,19 +819,21 @@ KStatusNotifierItemPrivate::KStatusNotifierItemPrivate(KStatusNotifierItem *item
 
 void KStatusNotifierItemPrivate::init(const QString &extraId)
 {
+    q->setAssociatedWidget(qobject_cast<QWidget *>(q->parent()));
+#ifdef QT_DBUS_LIB
     qDBusRegisterMetaType<KDbusImageStruct>();
     qDBusRegisterMetaType<KDbusImageVector>();
     qDBusRegisterMetaType<KDbusToolTipStruct>();
 
     statusNotifierItemDBus = new KStatusNotifierItemDBus(q);
-    q->setAssociatedWidget(qobject_cast<QWidget *>(q->parent()));
 
-    QDBusServiceWatcher *watcher = new QDBusServiceWatcher(s_statusNotifierWatcherServiceName,
+    QDBusServiceWatcher *watcher = new QDBusServiceWatcher(QString::fromLatin1(s_statusNotifierWatcherServiceName),
             QDBusConnection::sessionBus(),
             QDBusServiceWatcher::WatchForOwnerChange,
             q);
     QObject::connect(watcher, SIGNAL(serviceOwnerChanged(QString,QString,QString)),
                      q, SLOT(serviceChange(QString,QString,QString)));
+#endif
 
     //create a default menu, just like in KSystemtrayIcon
     QMenu *m = new QMenu(associatedWidget);
@@ -795,12 +865,16 @@ void KStatusNotifierItemPrivate::init(const QString &extraId)
     QAction *action = new QAction(q);
     action->setText(KStatusNotifierItem::tr("Quit"));
     action->setIcon(QIcon::fromTheme(QStringLiteral("application-exit")));
+    // cannot yet convert to function-pointer-based connect:
+    // some apps like kalarm or korgac have a hack to rewire the connection
+    // of the "quit" action to a own slot, and rely on the name-based slot to disconnect
+    // TODO: extend KStatusNotifierItem API to support such needs
     QObject::connect(action, SIGNAL(triggered()), q, SLOT(maybeQuit()));
     actionCollection.insert(QStringLiteral("quit"), action);
 
     id = title;
     if (!extraId.isEmpty()) {
-        id.append('_').append(extraId);
+        id.append(QLatin1Char('_')).append(extraId);
     }
 
     // Init iconThemePath to the app folder for now
@@ -811,19 +885,17 @@ void KStatusNotifierItemPrivate::init(const QString &extraId)
 
 void KStatusNotifierItemPrivate::registerToDaemon()
 {
+    bool useLegacy = false;
+#ifdef QT_DBUS_LIB
     qCDebug(LOG_KNOTIFICATIONS) << "Registering a client interface to the KStatusNotifierWatcher";
     if (!statusNotifierWatcher) {
-        statusNotifierWatcher = new org::kde::StatusNotifierWatcher(s_statusNotifierWatcherServiceName, QStringLiteral("/StatusNotifierWatcher"),
+        statusNotifierWatcher = new org::kde::StatusNotifierWatcher(QString::fromLatin1(s_statusNotifierWatcherServiceName), QStringLiteral("/StatusNotifierWatcher"),
                 QDBusConnection::sessionBus());
-        QObject::connect(statusNotifierWatcher, SIGNAL(StatusNotifierHostRegistered()),
-                         q, SLOT(checkForRegisteredHosts()));
-        QObject::connect(statusNotifierWatcher, SIGNAL(StatusNotifierHostUnregistered()),
-                         q, SLOT(checkForRegisteredHosts()));
     }
 
     if (statusNotifierWatcher->isValid()) {
         // get protocol version in async way
-        QDBusMessage msg = QDBusMessage::createMethodCall(s_statusNotifierWatcherServiceName,
+        QDBusMessage msg = QDBusMessage::createMethodCall(QString::fromLatin1(s_statusNotifierWatcherServiceName),
                                                           QStringLiteral("/StatusNotifierWatcher"),
                                                           QStringLiteral("org.freedesktop.DBus.Properties"),
                                                           QStringLiteral("Get"));
@@ -852,8 +924,10 @@ void KStatusNotifierItemPrivate::registerToDaemon()
         );
     } else {
         qCDebug(LOG_KNOTIFICATIONS) << "KStatusNotifierWatcher not reachable";
-        setLegacySystemTrayEnabled(true);
+        useLegacy = true;
     }
+#endif
+    setLegacySystemTrayEnabled(useLegacy);
 }
 
 void KStatusNotifierItemPrivate::serviceChange(const QString &name, const QString &oldOwner, const QString &newOwner)
@@ -863,20 +937,16 @@ void KStatusNotifierItemPrivate::serviceChange(const QString &name, const QStrin
         //unregistered
         qCDebug(LOG_KNOTIFICATIONS) << "Connection to the KStatusNotifierWatcher lost";
         setLegacyMode(true);
+#ifdef QT_DBUS_LIB
         delete statusNotifierWatcher;
         statusNotifierWatcher = nullptr;
+#endif
     } else if (oldOwner.isEmpty()) {
         //registered
         setLegacyMode(false);
     }
 }
 
-void KStatusNotifierItemPrivate::checkForRegisteredHosts()
-{
-    setLegacyMode(!statusNotifierWatcher ||
-                  !statusNotifierWatcher->property("IsStatusNotifierHostRegistered").toBool());
-}
-
 void KStatusNotifierItemPrivate::setLegacyMode(bool legacy)
 {
     if (legacy) {
@@ -890,7 +960,9 @@ void KStatusNotifierItemPrivate::setLegacyMode(bool legacy)
 
 void KStatusNotifierItemPrivate::legacyWheelEvent(int delta)
 {
+#ifdef QT_DBUS_LIB
     statusNotifierItemDBus->Scroll(delta, QStringLiteral("vertical"));
+#endif
 }
 
 void KStatusNotifierItemPrivate::legacyActivated(QSystemTrayIcon::ActivationReason reason)
@@ -898,7 +970,7 @@ void KStatusNotifierItemPrivate::legacyActivated(QSystemTrayIcon::ActivationReas
     if (reason == QSystemTrayIcon::MiddleClick) {
         emit q->secondaryActivateRequested(systemTrayIcon->geometry().topLeft());
     } else if (reason == QSystemTrayIcon::Trigger) {
-        q->activate();
+        q->activate(systemTrayIcon->geometry().topLeft());
     }
 }
 
@@ -909,13 +981,12 @@ void KStatusNotifierItemPrivate::setLegacySystemTrayEnabled(bool enabled)
         return;
     }
 
-
     if (enabled) {
-        if (!QSystemTrayIcon::isSystemTrayAvailable()) {
-            return;
-        }
         bool isKde = !qEnvironmentVariableIsEmpty("KDE_FULL_SESSION") || qgetenv("XDG_CURRENT_DESKTOP") == "KDE";
         if (!systemTrayIcon && !isKde) {
+            if (!QSystemTrayIcon::isSystemTrayAvailable()) {
+                return;
+            }
             systemTrayIcon = new KStatusNotifierLegacyIcon(associatedWidget);
             syncLegacySystemTrayIcon();
             systemTrayIcon->setToolTip(toolTipTitle);
@@ -1074,7 +1145,7 @@ void KStatusNotifierItemPrivate::maybeQuit()
 
 void KStatusNotifierItemPrivate::minimizeRestore()
 {
-    q->activate(QPoint(0, 0));
+    q->activate(systemTrayIcon ? systemTrayIcon->geometry().topLeft() : QPoint(0, 0));
 }
 
 void KStatusNotifierItemPrivate::hideMenu()
@@ -1102,16 +1173,17 @@ void KStatusNotifierItemPrivate::minimizeRestore(bool show)
     }
 }
 
+#ifdef QT_DBUS_LIB
 KDbusImageStruct KStatusNotifierItemPrivate::imageToStruct(const QImage &image)
 {
     KDbusImageStruct icon;
     icon.width = image.size().width();
     icon.height = image.size().height();
     if (image.format() == QImage::Format_ARGB32) {
-        icon.data = QByteArray((char *)image.bits(), image.byteCount());
+        icon.data = QByteArray((char *)image.bits(), image.sizeInBytes());
     } else {
         QImage image32 = image.convertToFormat(QImage::Format_ARGB32);
-        icon.data = QByteArray((char *)image32.bits(), image32.byteCount());
+        icon.data = QByteArray((char *)image32.bits(), image32.sizeInBytes());
     }
 
     //swap to network byte order if we are little endian
@@ -1133,13 +1205,15 @@ KDbusImageVector KStatusNotifierItemPrivate::iconToVector(const QIcon &icon)
     QPixmap iconPixmap;
 
     //if an icon exactly that size wasn't found don't add it to the vector
-    foreach (QSize size, icon.availableSizes()) {
+    const auto lstSizes = icon.availableSizes();
+    for (QSize size : lstSizes) {
         iconPixmap = icon.pixmap(size);
         iconVector.append(imageToStruct(iconPixmap.toImage()));
     }
 
     return iconVector;
 }
+#endif
 
 #include "moc_kstatusnotifieritem.cpp"
 #include "moc_kstatusnotifieritemprivate_p.cpp"
